import { defineComponent, PropType } from 'vue'
import styles from './index.module.less'
import {
  ActionSheet,
  Cell,
  CellGroup,
  DatetimePicker,
  Icon,
  Popup,
  Sticky,
  Image,
  List
} from 'vant'
import { formatterDate } from '@/helpers/utils'
import { goodsType, orderType } from '@/constant'

import iconTeacher from '@common/images/icon_teacher.png'
import request from '@/helpers/request'
import dayjs from 'dayjs'
import ColResult from '@/components/col-result'

export default defineComponent({
  name: 'list',
  props: {
    type: {
      type: String as PropType<'buy' | 'refund'>,
      default: 'buy'
    },
    height: {
      type: Number,
      default: 44
    }
  },
  data() {
    return {
      timeStatus: false,
      currentDate: new Date(),
      typeStatus: false,
      // 订单状态 WAIT_PAY 待支付 PAYING 支付中 PAID 已付款 CLOSE 已关闭 FAIL 支付失败 （多选用,分割）
      actions: [
        { name: '全部' },
        { name: '待支付', status: 'WAIT_PAY' },
        { name: '支付中', status: 'PAYING' },
        { name: '已付款', status: 'PAID' },
        { name: '已关闭', status: 'CLOSE' },
        { name: '支付失败', status: 'FAIL' }
      ],
      list: [],
      dataShow: true, // 判断是否有数据
      loading: false,
      finished: false,
      searchName: '全部',
      params: {
        status: '',
        page: 1,
        rows: 20
      }
    }
  },
  methods: {
    async getList() {
      try {
        const params = {
          ...this.params,
          searchDate: dayjs(this.currentDate).format('YYYY-MM')
        }
        const res = await request.post('/api-student/userOrder/page', {
          data: params
        })
        this.loading = false
        const result = res.data || {}
        // 处理重复请求数据
        if (this.list.length > 0 && result.pageNo === 1) {
          return
        }
        this.list = this.list.concat(result.rows || [])
        this.finished = result.pageNo >= result.totalPage
        this.params.page = result.pageNo + 1
        this.dataShow = this.list.length > 0
      } catch {
        this.dataShow = false
        this.finished = true
      }
    },
    onDetail(item: any) {
      this.$router.push({
        path: '/tradeDetail',
        query: {
          orderNo: item.orderNo,
          path: 'tradeRecord'
        }
      })
    },
    onConfirm(date: Date) {
      this.currentDate = date
      this.timeStatus = false
      this.onSearch()
    },
    onSelect(item: any) {
      this.params.status = item.status
      this.searchName = item.name
      this.onSearch()
    },
    onSearch() {
      this.dataShow = true
      this.loading = false
      this.finished = false
      this.list = []
      this.params.page = 1
      this.getList()
    }
  },
  render() {
    return (
      <div class={styles.tradeList}>
        <Sticky position="top" offsetTop={this.height}>
          <Cell
            center
            style={{ backgroundColor: '#F7F8F9' }}
            v-slots={{
              title: () => (
                <div
                  class={styles.searchTime}
                  onClick={() => {
                    this.timeStatus = true
                  }}
                >
                  <span>
                    {(this as any).$filters.dateFormat(
                      this.currentDate,
                      'YYYY-MM'
                    )}
                  </span>
                  <Icon
                    classPrefix="iconfont"
                    name="down"
                    size={12}
                    color="var(--van-primary)"
                  />
                </div>
              ),
              value: () => (
                <div
                  class={styles.searchType}
                  onClick={() => {
                    this.typeStatus = true
                  }}
                >
                  <span>{this.searchName}</span>
                  <Icon
                    classPrefix="iconfont"
                    name="down"
                    size={12}
                    color="var(--van-primary)"
                  />
                </div>
              )
            }}
          ></Cell>
        </Sticky>
        {this.dataShow ? (
          <List
            v-model:loading={this.loading}
            finished={this.finished}
            finishedText=" "
            class={[styles.list]}
            onLoad={this.getList}
          >
            {this.list.map((item: any) => (
              <CellGroup
                border={false}
                onClick={() => {
                  this.onDetail(item)
                }}
              >
                <Cell
                  title={dayjs(item.createTime).format('YYYY-MM-DD HH:mm')}
                  value={orderType[item.status]}
                  valueClass={styles.tradeType}
                />
                <Cell
                  v-slots={{
                    // icon: () => (
                    //   <Image
                    //     class={styles.tradeLogo}
                    //     src={iconTeacher}
                    //     fit="cover"
                    //   />
                    // ),
                    title: () => (
                      <div class={styles.title}>
                        <span>{item.orderName}</span>
                        <span class={styles.desc}>
                          {goodsType[item.orderType]}
                        </span>
                      </div>
                    ),
                    default: () => (
                      <div class={styles.content}>
                        <span class={styles.price}>
                          ￥
                          {(this as any).$filters.moneyFormat(item.actualPrice)}
                        </span>
                        {/* <span class={styles.num}>x1</span> */}
                      </div>
                    )
                  }}
                />
              </CellGroup>
            ))}
          </List>
        ) : (
          <ColResult
            btnStatus={false}
            classImgSize="SMALL"
            tips="暂无购买记录"
          />
        )}

        <Popup show={this.timeStatus} position="bottom" round closeOnPopstate>
          <DatetimePicker
            type="year-month"
            v-model={this.currentDate}
            formatter={formatterDate}
            onCancel={() => {
              this.timeStatus = false
            }}
            onConfirm={this.onConfirm}
          />
        </Popup>

        <ActionSheet
          v-model:show={this.typeStatus}
          actions={this.actions}
          closeOnClickAction
          cancelText="取消"
          onSelect={this.onSelect}
          onCancel={() => {
            this.typeStatus = false
          }}
        />
      </div>
    )
  }
})
